import os
import sys

import django
from django.conf import settings
from django.test.utils import get_runner


APP_DIR = os.path.abspath(os.path.dirname(__file__))

SETTINGS_DICT = {
    'TEMPLATES': [{
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
    }],
    'STATICFILES_FINDERS': [
        'django.contrib.staticfiles.finders.FileSystemFinder',
        'django.contrib.staticfiles.finders.AppDirectoriesFinder',
        'remote_finder.RemoteFinder',
    ],
    'REMOTE_FINDER_RESOURCES': [
        ('js/jquery.min.js', 'https://code.jquery.com/jquery-3.3.1.min.js', 'sha1:0dc32db4aa9c5f03f3b38c47d883dbd4fed13aae'),
    ],
    'REMOTE_FINDER_CACHE_DIR': os.path.join(APP_DIR, 'tests', 'cache'),
}

def run_tests():
    settings.configure(**SETTINGS_DICT)
    django.setup()
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=2, interactive=True)
    failures = test_runner.run_tests(["tests"])
    sys.exit(bool(failures))


if __name__ == "__main__":
    run_tests()



import hashlib
import logging
from urllib.request import urlopen

from django.core.exceptions import ImproperlyConfigured
from django.core.files.base import ContentFile
from django.core.files.storage import FileSystemStorage
from django.contrib.staticfiles.finders import BaseFinder
from django.contrib.staticfiles.utils import matches_patterns
from django.conf import settings

logger = logging.getLogger(__name__)

hash_func_map = {
    'md5': hashlib.md5,
    'sha1': hashlib.sha1,
    'sha224': hashlib.sha224,
    'sha256': hashlib.sha256,
    'sha384': hashlib.sha384,
    'sha512': hashlib.sha512,
}


class _ResourceInfo:
    hash_verified = False

    def __init__(self, url, hash_func, expected_digest):
        self.url = url
        self.hash_func = hash_func
        self.expected_digest = expected_digest


class RemoteFinder(BaseFinder):
    def __init__(self):
        self.always_verify = getattr(settings, "REMOTE_FINDER_ALWAYS_VERIFY", False)
        self.cache_dir = getattr(settings, "REMOTE_FINDER_CACHE_DIR", None)
        if not self.cache_dir:
            raise ImproperlyConfigured("settings.REMOTE_FINDER_CACHE_DIR must point to a cache directory.")
        self.storage = FileSystemStorage(self.cache_dir)
        try:
            resources_setting = settings.REMOTE_FINDER_RESOURCES
        except AttributeError:
            logger.warning("RemoteFinder is enabled, but settings.REMOTE_FINDER_RESOURCES is not defined.")
            resources_setting = ()
        if not isinstance(resources_setting, (list, tuple)):
            raise ImproperlyConfigured("settings.REMOTE_FINDER_RESOURCES must be a list or tuple")
        resources = {}
        for resource in resources_setting:
            try:
                path, url, cksm = resource
            except ValueError:
                raise ImproperlyConfigured("Each item in settings.REMOTE_FINDER_RESOURCES must be a tuple of three elements (path, url, cksm).")
            try:
                hash_type, expected_hexdigest = cksm.split(':')
            except ValueError:
                raise ImproperlyConfigured("RemoteFinder checksum `%s` is not in `hash_type:hexdigest` format." % cksm)
            try:
                hash_func = hash_func_map[hash_type]
            except KeyError:
                raise ImproperlyConfigured("RemoteFinder: hash type `%s` unknown" % hash_type)
            try:
                expected_digest = bytearray.fromhex(expected_hexdigest)
            except ValueError:
                raise ImproperlyConfigured("Cannot parse hex string in settings.REMOTE_FINDER_RESOURCES: `%s`" % expected_hexdigest)
            if len(expected_digest) != hash_func().digest_size:
                raise ImproperlyConfigured("settings.REMOTE_FINDER_RESOURCES: %s digest expected %d bytes but %d provided: `%s`" % (hash_type, hash_func().digest_size, len(expected_digest), expected_hexdigest))
            resources[path] = _ResourceInfo(url, hash_func, expected_digest)
        self.resources = resources

    def find(self, path, all=False):
        try:
            resource_info = self.resources[path]
        except KeyError:
            return []
        self.fetch(path, resource_info)
        match = self.storage.path(path)
        if all:
            return [match]
        else:
            return match

    def fetch(self, path, resource_info):
        if self.storage.exists(path):
            # check to see if the hash has already been verified in the
            # lifetime of this process
            if resource_info.hash_verified and not self.always_verify:
                return

            # verify the hash
            f = self.storage.open(path)
            try:
                content = f.read()
            finally:
                f.close()
            digest = resource_info.hash_func(content).digest()
            if digest == resource_info.expected_digest:
                resource_info.hash_verified = True
                return

            # hash verification failed, so delete it from storage and
            # re-download the file
            logger.info("Hash verification failed, so deleting %s from storage", path)

            # The following line does /not/ raise an exception if the file is
            # already deleted, which is desirable for us as it prevents an
            # error in the case of a race condition.
            self.storage.delete(path)

        # download the file
        logger.info("Downloading %s", resource_info.url)
        f = urlopen(resource_info.url)
        try:
            content = f.read()
        finally:
            f.close()

        # check its hash
        digest = resource_info.hash_func(content).digest()
        if digest != resource_info.expected_digest:
            raise RuntimeError(
                "Digest for %s does not match expected value given in settings"
                ".REMOTE_FINDER_RESOURCES" % resource_info.url
            )

        # save it
        name = self.storage.save(path, ContentFile(content))
        if name == path:
            resource_info.hash_verified = True
        else:
            logger.warning("Save failed: %r != %r", name, path)

    def list(self, ignore_patterns):
        for path, resource_info in self.resources.items():
            if matches_patterns(path, ignore_patterns):
                continue
            self.fetch(path, resource_info)
            yield path, self.storage



import os
import shutil

from django.conf import settings
from django.contrib.staticfiles import finders
from django.core.exceptions import ImproperlyConfigured
from django.template import Context, Template
from django.test import modify_settings, SimpleTestCase


class FinderTests(SimpleTestCase):
    def setUp(self):
        super().setUp()
        finders.get_finder.cache_clear()
        shutil.rmtree(settings.REMOTE_FINDER_CACHE_DIR, ignore_errors=True)

    def test_missing_cache_dir(self):
        msg = "settings.REMOTE_FINDER_CACHE_DIR must point to a cache directory."
        with self.settings():
            del settings.REMOTE_FINDER_CACHE_DIR
            with self.assertRaisesMessage(ImproperlyConfigured, msg):
                finder = finders.get_finder('remote_finder.RemoteFinder')

    def test_missing_resources(self):
        msg = "settings.REMOTE_FINDER_RESOURCES must point to a cache directory."
        with self.settings():
            del settings.REMOTE_FINDER_RESOURCES
            with self.assertLogs('remote_finder', level='WARN') as cm:
                finder = finders.get_finder('remote_finder.RemoteFinder')
            self.assertEqual(cm.output, [
                'WARNING:remote_finder:RemoteFinder is enabled, but '
                'settings.REMOTE_FINDER_RESOURCES is not defined.'
            ])

    def test_finder_find(self):
        finder = finders.get_finder('remote_finder.RemoteFinder')
        path = finder.find('js/jquery.min.js')
        self.assertEqual(
            path,
            os.path.join(settings.REMOTE_FINDER_CACHE_DIR, 'js', 'jquery.min.js')
        )
        # No match
        path = finder.find('js/jquery.js')
        self.assertEqual(path, [])

    def test_finder_list(self):
        finder = finders.get_finder('remote_finder.RemoteFinder')
        listed = list(finder.list([]))
        self.assertEqual(len(listed), 1)
        self.assertEqual(listed[0][0], 'js/jquery.min.js')
        file_path = os.path.join(settings.REMOTE_FINDER_CACHE_DIR, 'js', 'jquery.min.js')
        self.assertTrue(os.path.exists(file_path))

    def test_render(self):
        t = Template('''{% load static %}<script src="{% static 'js/jquery.min.js' %}">''')
        result = t.render(Context({}))
        self.assertEqual(result, '<script src="js/jquery.min.js">')

    @modify_settings(REMOTE_FINDER_RESOURCES={
        'append': [(
            'js/jquery.min2.js',
            'https://code.jquery.com/jquery-3.3.1.min.js',
            'sha1:2885e21e33426ab1c8c8e587b038e713521cc3da'
        )],
    })
    def test_bad_checksum(self):
        finder = finders.get_finder('remote_finder.RemoteFinder')
        msg = (
            'Digest for https://code.jquery.com/jquery-3.3.1.min.js does not '
            'match expected value given in settings.REMOTE_FINDER_RESOURCES'
        )
        with self.assertRaisesMessage(RuntimeError, msg):
            path = finder.find('js/jquery.min2.js')
